
#ifndef UNIT_TEST
#include <Arduino.h>
#endif
#include <stdlib.h>
#include <Adafruit_MQTT_Client.h>
#include <Adafruit_MQTT.h>       
#include <BlynkSimpleEsp8266.h>
#define BLYNK_PRINT Serial
#define BLYNK_MAX_SENDBYTES 1200 // Allows for sending longer emails, if needed

// All the input and output pins that we're using
#define EMERGENCY D4               //digital pin 4
#define ALCOHOL A0                 //analog pin0
#define MICROPHONE D3              //digital pin 3
#define MAX_VALUE 300              //volume limit
#define EMAIL "your email"

String soundWarning = "The driver volume has exceeded the high volume limit:";
String AlcoholWarning = "the driver has a high alcohol smell:";

// Init variables for the loop section

int AlcValue;       //alcohol level
const int SAMPLE_TIME = 10;
unsigned long millisCurrent;
unsigned long millisLast = 0;
unsigned long millisElapsed = 0;
int sampleBufferValue =0;

// WiFi credentials
char ssid[] = "your Wifi network name";            
char pass[] = "your Wifi network password";      

// Blynk authorization code
char auth[] = "your Blynk authorization code";

// Adafruit IO information and setup
#define AIO_SERVER "io.adafruit.com"
#define AIO_SERVERPORT 1883
#define AIO_USERNAME "AdafruitIO username"                // AdafruitIO username
#define AIO_KEY "AdafruitIO key"                       //AdafruitIO key 

void connect();
WiFiClient client;
const char MQTT_CLIENTID[] = AIO_KEY __DATE__ __TIME__;
const char MQTT_USERNAME[] = AIO_USERNAME;
const char MQTT_PASSWORD[] = AIO_KEY;
const char MQTT_SERVER[] = AIO_SERVER;
Adafruit_MQTT_Client mqtt(&client, MQTT_SERVER, AIO_SERVERPORT, MQTT_CLIENTID, MQTT_USERNAME, MQTT_PASSWORD);

// EMERGENCY feed
const char EMERGENCY_FEED[] = AIO_USERNAME "/feeds/emergency";
Adafruit_MQTT_Publish emergency = Adafruit_MQTT_Publish(&mqtt, EMERGENCY_FEED);
int emergencyStatus = 1;

// EMERGENCY location feed
const char LOCATION_FEED[] = AIO_USERNAME "/feeds/location/csv";
Adafruit_MQTT_Publish loc = Adafruit_MQTT_Publish(&mqtt, LOCATION_FEED);


// GPS location variables
float latitude = 0;
float longitude = 0;
int alt = 0;
float s = 0;

// init boolean variables
boolean isSafe = true;
boolean isSober = true;
boolean isQuiet = true;

void setup() {
  Serial.begin(9600);
  pinMode(ALCOHOL,INPUT);             //init the alcohol sensor
  pinMode(MICROPHONE,INPUT);          //init the Sound sensor
  pinMode(EMERGENCY, INPUT);          // Init the emergency button
  Blynk.begin(auth, ssid, pass);      // Activate Blynk
 
  // Setting connection to Adafruit IO
  WiFi.begin(ssid, pass);
  // wait for WiFi connection
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
    delay(10);
  }
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
  Serial.println();
  // connect to adafruit io
  connect();

}

void loop() {
  Blynk.run();
  
  //Alcohol check
  AlcValue= analogRead(ALCOHOL);    //reading alcohol level
  //Serial.println(AlcValue);      
  if(AlcValue>550 && isSober)       //checking if level is high & if already sent email
  {
    AlcoholWarning+= AlcValue;
    Blynk.email(EMAIL,"subject:Alcohol level is high", AlcoholWarning); // Send ourselves the email
    Serial.println("sent alcohol email");
    isSober = false;
  }

  //Volume check
  millisCurrent = millis();
  millisElapsed = millisCurrent - millisLast;

  if (digitalRead(MICROPHONE) == LOW) {
    sampleBufferValue++;
  }

  if (millisElapsed > SAMPLE_TIME) {                          //10 millis has passed
    if((sampleBufferValue > MAX_VALUE)  && isQuiet){         //is the volume too high?
      soundWarning+= (sampleBufferValue);
      Blynk.email(EMAIL,"subject:High Volume driving", soundWarning); // Send ourselves the email
      isQuiet = false;   
    }
    sampleBufferValue = 0;
    millisLast = millisCurrent;
  }

  // Goes off when Emergency button is pressed
  if (digitalRead(EMERGENCY) == LOW) { // Reed switch is closed
    if (!isSafe) { // This means we just switched to paking
      isSafe = true;

      char buff[10];
      String msg = "";
      // Parse latitude and longitude to the string that will be emailed to us
      msg += "Your EMERGENCY coordinates are:\nLat: ";
      dtostrf(latitude, 4, 6, buff); // Parse latitude to a char buffer
      msg += buff;
      msg += "\nLon: ";
      dtostrf(longitude, 4, 6, buff); // Parse longitude to a char buffer
      msg += buff;
      msg += "\nInsert lat and lon coordinates at: https://www.google.com/maps.";

      Blynk.email(EMAIL,"subject:Emergency button clicked", msg); // Send ourselves the email

      // Send EMERGENCY data to Adafruit IO
      emergencyStatus = 1;
      if (!emergency.publish(emergencyStatus)) Serial.println("Failed");
      else Serial.println("Sent OK! " + String(emergencyStatus));

      // Parse GPS location data into CSV format
      char GPSBuffer[30]; // Combine Latitude, Longitude, Altitude into a buffer of size X
      char *p = GPSBuffer; // Create a buffer to store GPS information to upload to Adafruit IO

      dtostrf(s, 1, 4, p); // Convert speed to a String Variable and add it to the buffer
      p += strlen(p);
      p[0] = ','; p++;
      
      dtostrf(latitude, 3, 6, p); // Convert latitude to a String variable and add it to the buffer
      p += strlen(p);
      p[0] = ','; p++;
      
      dtostrf(longitude, 3, 6, p); // Convert longitude to a String variable and add it to the buffer
      p += strlen(p);
      p[0] = ','; p++;
      int altAsInt = alt;
      
      dtostrf(alt, 1, 0, p); // Convert altitude to a String variable and add it to the buffer
      p += strlen(p);
      
      p[0] = 0; // null terminate, end of buffer array

      // Send GPS location data to Adafruit IO
      if (!loc.publish(GPSBuffer)) Serial.println("Failed");
      else{
        Serial.print("Sent OK! ");
        Serial.println(GPSBuffer);
      }

    }
  } else {
    if (isSafe) {
      isSafe = false;
      // Send data to Adafruit IO
      emergencyStatus = 0;
      if (!emergency.publish(emergencyStatus)) Serial.println("Failed");
      else Serial.println("Sent OK! " + String(emergencyStatus));
    }
  }
}

// This function gets us our GPS coordinates
BLYNK_WRITE(V0) {
  latitude = param[0].asFloat();
  longitude = param[1].asFloat();
  alt = param[2].asInt();
  s = param[3].asFloat();
}

void connect() {
  Serial.print("Connecting to Adafruit IO... ");
  int8_t ret;
  while ((ret = mqtt.connect()) != 0) {
    switch (ret) {
      case 1: Serial.println("Wrong protocol"); break;
      case 2: Serial.println("ID rejected"); break;
      case 3: Serial.println("Server unavail"); break;
      case 4: Serial.println("Bad user/pass"); break;
      case 5: Serial.println("Not authed"); break;
      case 6: Serial.println("Failed to subscribe"); break;
      default: Serial.println("Connection failed"); break;
    }
    if (ret >= 0) {
      mqtt.disconnect();
    }
    Serial.println("Retrying connection...");
    delay(5000);
  }
  Serial.println("Adafruit IO Connected!");
}
